<?php

namespace App\Controllers;

use App\Models\User;

use Core\Validation;
use Firebase\JWT\JWT;
use Firebase\JWT\Key;

class AuthController{

    public function register(){
        $data = json_decode(file_get_contents("php://input"), true);

        if(!$data){
            $data = $_POST;
        }

        $registerErrors = Validation::validateRegister($data);

        if( $registerErrors !== []){
            return json_encode($registerErrors);
        }

        if(Validation::userExists($data['email'])){
            header( "HTTP/1.0 400 Bad Request" );
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(["error"=> "user already exists"]);
        }

        $user = new User();

        $token = $user::store($data);

        if($token !== false){
            header( "HTTP/1.0 201 Created" );
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(["token"=>$token]);
        }else{
            header('HTTP/1.0 500 Internal Server Error');
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(['error' => 'Error accessing the data base.']);
        }


    }


    public function login(){
        $data = json_decode(file_get_contents("php://input"), true);

        if(!$data){
            $data = $_POST;
        }

        $loginErrors = Validation::validateLogin($data);

        if( $loginErrors !== []){
            return json_encode($loginErrors);
        }

        $user = User::find($data['email']);

        if(!$user){
            header('HTTP/1.0 404 Not Found');
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode([ "error"=>"User not found" ]);
        }

        if(!password_verify($data['password'], $user['password'])){
            header('HTTP/1.0 403 Forbidden');
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode([ "error"=>"Wrong password" ]);
        }


        $savedUser = new User();

        $token = $savedUser::save($data['email']);

        if($token !== false){

            header( "HTTP/1.0 200 Ok" );
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(["token"=>$token]);
        }else{
            header('HTTP/1.0 500 Internal Server Error');
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(['error' => 'Error accessing the data base.']);
        }



    }

    public function logout(){

        if(!isset(getallheaders()['Authorization'])){
            header( "HTTP/1.0 401 Unauthorized" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode(["message" => "Unauthorized"]);
        }

        $authHeader = getallheaders()['Authorization'] ?? '';
        $token = str_replace('Bearer ', '', $authHeader);

        if(Validation::validateToken($token) !== true){
            header( "HTTP/1.0 401 Unauthorized" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return(["message" => "Unauthorized"]);
        }

        if(User::destroyToken(JWT::decode($token, new Key("user_key", 'HS256'))) !== false){
            header( "HTTP/1.0 200 Ok" );
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(["message" => "Logged out"]);
        }else{
            header('HTTP/1.0 500 Internal Server Error');
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(['error' => 'Error accessing the data base.']);
        }

    }

}
