<?php

namespace App\Controllers;

use App\Models\Order;

use Core\Validation;

class OrderController{

    public function index(){
        if(!isset(getallheaders()['Authorization'])){
            header( "HTTP/1.0 401 Unauthorized" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode(["message" => "Unauthorized"]);

        }

        $authHeader = getallheaders()['Authorization'];
        $token = str_replace('Bearer ', '', $authHeader);

        if(Validation::validateToken($token) !== true){
            header( "HTTP/1.0 401 Unauthorized" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode(["message" => "Unauthorized"]);
        }

        $orders = Order::all($token);

        if($orders !== false){
            header( "HTTP/1.0 200 Ok" );
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode($orders);
        }else{
            header('HTTP/1.0 500 Internal Server Error');
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(['error' => 'Error accessing the data base.']);
        }


    }

    public function store(){
        if(!isset(getallheaders()['Authorization'])){
            header( "HTTP/1.0 401 Unauthorized" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode(["message" => "Unauthorized"]);

        }

        $authHeader = getallheaders()['Authorization'];
        $token = str_replace('Bearer ', '', $authHeader);

        if(Validation::validateToken($token) !== true ){
            header( "HTTP/1.0 401 Unauthorized" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode(["message" => "Unauthorized"]);
        }

        $data = json_decode(file_get_contents("php://input"), true);

        if(!$data){
            $data = $_POST;
        }

        $createErrors = Validation::validateOrder($_POST);

        if($createErrors !== []){
            header( "HTTP/1.0 400 Bad Request" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode($createErrors);
        }

        $order = new Order();

        $orderResponse = $order::store($token);

        if($orderResponse === null){
            header( "HTTP/1.0 400 Bad Request" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode(["message" => "not enough units of the product in stock"]);
        }

        if($orderResponse === false){
            header('HTTP/1.0 404 Not Found');
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(['error' => 'Product not found.']);
        }

        header( "HTTP/1.0 201 Created" );
        header( "Content-Type: application/json; charset=UTF-8" );
        return json_encode(["message" => "Order created successfully"]);


    }

    public function delete($id){
        if(!isset(getallheaders()['Authorization'])){
            header( "HTTP/1.0 401 Unauthorized" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode(["message" => "Unauthorized"]);

        }

        $authHeader = getallheaders()['Authorization'];
        $token = str_replace('Bearer ', '', $authHeader);

        if(Validation::validateToken($token) !== true ){
            header( "HTTP/1.0 401 Unauthorized" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode(["message" => "Unauthorized"]);
        }

        $orderResponse = Order::delete($id, $token);

        if($orderResponse === null){
            header( "HTTP/1.0 404 Not Found" );
            header( "Content-Type: application/json; charset=UTF-8" );
            return json_encode(["message" => "Order not found"]);
        }

        if($orderResponse !== false){
            header('HTTP/1.0 200 Ok');
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(['message' => 'Order deleted successfully']);
        }else{
            header('HTTP/1.0 500 Internal Server Error');
            header("Content-Type: application/json; charset=UTF-8");
            return json_encode(['error' => 'Error in the data base.']);
        }

    }


}