<?php

namespace App\Models;

use App\Models\Item;
use Core\Model;
use App\Models\User;

use Firebase\JWT\JWT;
use Firebase\JWT\Key;
use mysql_xdevapi\Exception;
use PDO;
use PDOException;



class Order extends Model {

    public static function all($token){
        try{
            $db = self::connection();

            $decodedToken = JWT::decode($token, new Key("user_key", 'HS256'));

            $stmt = $db->prepare("SELECT items.id, items.name, items.description, items.price, orders.units 
                                        FROM items
                                        JOIN orders ON items.id = orders.item_id
                                        JOIN users ON users.id = orders.user_id
                                        WHERE users.email = :email");

            $stmt->bindParam(":email", $decodedToken->iss);
            $stmt->execute();

            $items = $stmt->fetchAll(PDO::FETCH_ASSOC);

            $price = 0;

            foreach ($items as $item){
                $price = $price + $item["price"]*$item["units"];
            }

            if($decodedToken->aud === "Malaga"){
                $discount = "discount applied";
                $price = $price - $price * 0.20;

            }else{
                $discount = "No discount applied";
            }

            $response = [
                "total price" => round($price, 2),
                "discount" => $discount,
                "data" => $items
            ];

            return $response;
        }catch (PDOException $e){
            return false;
        }

    }

    public static function find($userId, $itemId){
        try{
            $db = self::connection();
            $stmt = $db->prepare("SELECT * FROM orders WHERE user_id = :user_id AND item_id = :item_id");
            $stmt->bindParam(":user_id", $userId);
            $stmt->bindParam(":item_id", $itemId);
            $stmt->execute();
            return $stmt->fetch();
        }catch (PDOException $e){
            return $e->getMessage();
        }


    }

    public static function updateUnits($units, $userId, $itemId){
        try{
            $db = self::connection();

            $stmt = $db->prepare("UPDATE orders SET units = :units WHERE user_id = :user_id AND item_id = :item_id");
            $stmt->bindParam( ":units", $units);
            $stmt->bindParam(":user_id", $userId);
            $stmt->bindParam(":item_id", $itemId);
            $stmt->execute();
        } catch (PDOException $e){
            return $e->getMessage();
        }
    }

    public static function store($token){

        try{
            $db = self::connection();

            $decodedToken = JWT::decode($token, new Key("user_key", 'HS256'));

            $user = User::find($decodedToken->iss);

            $oldItem = Item::findByName($_POST["name"]);

            if($oldItem === false){
                return false;
            }

            if($oldItem["stock"] < $_POST["units"]){
                return null;
            }

            $existentOrder = Order::find($user["id"], $oldItem["id"]);


            if($existentOrder !== false){
                $newUnits = $existentOrder["units"] + $_POST["units"];
                Order::updateUnits($newUnits, $user["id"], $oldItem["id"]);

                $newStock = $oldItem["stock"] - $_POST["units"];
                return Item::updateStock($newStock, $oldItem["id"]);
            }

            $stmt = $db->prepare("INSERT INTO orders VALUES (:user_id, :item_id, :units) ");

            $stmt->bindParam(":user_id", $user["id"]);
            $stmt->bindParam(":item_id", $oldItem["id"]);
            $stmt->bindParam(":units", $_POST["units"]);
            $stmt->execute();

            $newStock = $oldItem["stock"] - $_POST["units"];
            return Item::updateStock($newStock, $oldItem["id"]);

        }catch (PDOException $e){
            return $e->getMessage();
        }

    }


    public static function delete($id,$token){
        try{
            $db = self::connection();

            $decodedToken = JWT::decode($token, new Key("user_key", 'HS256'));

            $user = User::find($decodedToken->iss);
            $itemInt = intval($id[0]);

            $oldOrder = Order::find($user["id"], $itemInt);

            if($oldOrder === false){
                return null;
            }

            $stmt = $db->prepare("DELETE FROM orders WHERE user_id = :user_id AND item_id = :item_id");

            $stmt->bindParam(":user_id", $user["id"]);
            $stmt->bindParam(":item_id", $itemInt);
            $stmt->execute();

            $oldItem = Item::find($itemInt);

            $newStock = $oldItem["stock"]+$oldOrder["units"];

            $result = Item::updateStock($newStock, $itemInt);

            return $result;
        }catch (PDOException $e){
            return false;
        }


    }

}