<?php

namespace App\Models;

use Core\Model;
use Firebase\JWT\JWT;
use Firebase\JWT\Key;

use PDO;
use PDOException;

class User extends Model{

    public static function store($data){
        try{
            $db = self::connection();

            $key = 'user_key';
            $payload = [
                'iss' => $data['email'],
                'aud' => $data['city'],
            ];

            $jwt = JWT::encode($payload, $key, 'HS256');

            $hashedPassword = password_hash($data["password"], PASSWORD_DEFAULT);

            $stmt = $db->prepare("INSERT INTO users (name, email, password, city, token) VALUES (:name, :email, :password, :city, :token)");
            $stmt->bindParam(":name", $data["name"], PDO::PARAM_STR);
            $stmt->bindParam(":email", $data["email"], PDO::PARAM_STR);
            $stmt->bindParam(":password", $hashedPassword);
            $stmt->bindParam(":city", $data["city"], PDO::PARAM_STR);
            $stmt->bindParam(":token", $jwt, PDO::PARAM_STR);
            $stmt->execute();

            return $jwt;

        } catch (PDOException $e) {
            return false;
        }


    }

    public static function save($email){
        try{
            $db = self::connection();

            $user = User::find($email);

            $key = 'user_key';
            $payload = [
                'iss' => $email,
                'aud' => $user['city'],
            ];

            $jwt = JWT::encode($payload, $key, 'HS256');

            $stmt  = $db->prepare("UPDATE users SET token=:token WHERE email = :email");
            $stmt->bindParam(":email", $email, PDO::PARAM_STR);
            $stmt->bindParam(":token", $jwt, PDO::PARAM_STR);
            $stmt->execute();

            return $jwt;

        } catch (PDOException $e) {
            return false;
        }

    }

    public static function find($email){
        try{
            $db = self::connection();
            $stmt = $db->prepare("SELECT * FROM users WHERE email = :email");
            $stmt->bindParam(":email", $email, PDO::PARAM_STR);
            $stmt->execute();
            return $stmt->fetch();
        }catch (PDOException $e){
            return false;
        }

    }

    static function destroyToken($token){
        try{
            $db = self::connection();
            $stmt = $db->prepare("Update users SET token = NULL WHERE email = :email");
            $stmt->bindParam(":email", $token->iss, PDO::PARAM_STR);
            return $stmt->execute();
        } catch (PDOException $e) {
            return false;
        }

    }
}
