<?php

namespace Core;

use App\Models\User;
use App\Models\Item;

use Firebase\JWT\JWT;
use Firebase\JWT\Key;

class Validation{

    public static function test_input($data): string{
        $data = trim($data);
        $data = stripslashes($data);
        $data = htmlspecialchars($data);
        return $data;
    }


    static function validateToken($token)
    {
        $decodedToken = JWT::decode($token, new Key("user_key", 'HS256'));

        $user = User::find($decodedToken->iss);

        if($user['token'] !== $token){
            return false;
        }
        return true;
    }

    static function validateRegister($data)
    {
        $errors = array();

        if(empty($data['email'])){
            $errors['email'] = "Email in required";
        }else{
            $email = Validation::test_input($data['email']);
            if(!preg_match("/[a-zA-Z0-9]@[a-zA-Z0-9._-]+\.[a-zA-Z]{2,3}$/", $email)){
                $errors['email'] = "Invalid Email";
            }
        }

        if(empty($data['password'])){
            $errors['password'] = "Password is required";
        }

        if(empty($data['name'])){
            $errors['name'] = "Name is required";
        }

        if(empty($data['city'])){
            $errors['city'] = "City is required";
        }

        return $errors;
    }

    static function validateLogin($data){
        $errors = array();

        if(empty($data['email'])){
            $errors['email'] = "Email in required";
        }else{
            $email = Validation::test_input($data['email']);
            if(!preg_match("/[a-zA-Z0-9]@[a-zA-Z0-9._-]+\.[a-zA-Z]{2,3}$/", $email)){
                $errors['email'] = "Invalid Email";
            }
        }

        if(empty($data['password'])){
            $errors['password'] = "Password is required";
        }

        return $errors;
    }

    static function userExists($email){
        $user = User::find($email);

        if($user === false){
            return $user;
        }
        return true;
    }

    static function validateOrder($data){
        $errors = array();

        if(empty($data['name'])){
            $errors['name'] = "Name is required";
        }

        if(empty($data['units'])){
            $errors['units'] = "Units are required";
        }

        return $errors;
    }


}
